/*
 * This code is made available under version 3 of the
 * GNU GENERAL PUBLIC LICENSE. See the file LICENSE in this
 * distribution for details.
 * 
 * Copyright 2008 Eric Bodden
 */

package ca.mcgill.sable.racer;

/**
 * The 'exclusive' state in the racer algorithm. It is parameterized with
 * the thread t to which it is exclusive.
 *
 * @author Eric Bodden
 */
public class ExclusiveState extends State {

	/**
	 * The parameter thread t. 
	 */
	protected Thread t;

	/**
	 * @param t the parameter thread
	 * @param predecessorState the predecessor state
	 */
	public ExclusiveState(Thread t, VirginState predecessorState) {
		super(predecessorState);
		this.t = t;
	}
	
	/**
	 * {@inheritDoc}
	 */
	protected State newStateOnRead(Thread t) {
		if(t==this.t) {
			return this;
		} else {
			return new SharedState(this);
		}		
	}
	
	/**
	 * {@inheritDoc}
	 */
	protected State newStateOnWrite(Thread t) {
		if(t==this.t) {
			return new ModifiedState(t, this);
		} else {
			return new SharedModifiedState(this);
		}		
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "exclusive "+t.getName();
	}
	

}
