/*
 * This code is made available under version 3 of the
 * GNU GENERAL PUBLIC LICENSE. See the file LICENSE in this
 * distribution for details.
 * 
 * Copyright 2008 Eric Bodden
 */

package ca.mcgill.sable.racer;


/**
 * The exclusive-modified state from the racer algorithm.
 * A field is in this state when it has been written to but only
 * been accessed by a single thread.
 *
 * @author Eric Bodden
 */
public class ModifiedState extends State {

	/**
	 * The thread that accessed the field to which this state belongs. 
	 */
	protected Thread t;

	/**
	 * @param t the parameter thread
	 * @param predecessorState the predecessor state
	 */
	public ModifiedState(Thread t, State predecessorState) {
		super(predecessorState);
		this.t = t;
	}
	
	/**
	 * {@inheritDoc}
	 */
	protected State newStateOnRead(Thread t) {
		if(t==this.t) {
			return this;
		} else {
			return new SharedModifiedState(this);
		}		
	}
	
	/**
	 * {@inheritDoc}
	 */
	protected State newStateOnWrite(Thread t) {
		if(t==this.t) {
			return this;
		} else {
			return new SharedModifiedState(this);
		}		
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		return "modified "+t.getName();
	}
	
}
