package abc.da.ast;

import java.util.Set;

import polyglot.ast.ClassMember;
import polyglot.ast.Node;
import polyglot.types.SemanticException;
import polyglot.visit.ContextVisitor;

/**
 * The declaration of an advice dependency in source code.
 * An advice dependency declaration consists of a non-empty set
 * of strong advice names and a set of weak advice names.
 * Each advice name may have a vector of variables associated with it.
 * This vector must have the same length as the unique advice with that name
 * has parameters. (The returning/throwing parameter is taken as last parameter.)
 * Any variable can be {@link #WILDCARD}, which has the same meaning as a
 * fresh variable name.
 *  
 * @author Eric Bodden
 */
public interface AdviceDependency extends ClassMember {

	public static final String WILDCARD = "*";

	/**
	 * Checks that if two positions refer to the same variable, the advice formals
	 * at those positions are cast-convertible. 
	 * For primitive types we give a warning, that their variables will have no effect
	 * (if the variable at that position is not the {@link #WILDCARD}).
	 */
	public Node typeCheckAdviceParams(ContextVisitor visitor) throws SemanticException;

	/**
	 * Returns the set of all names of strong and weak advice in this dependency declaration.
	 */
	public Set<String> allAdviceNames();

}
