package abc.da.ast;

import java.util.List;
import java.util.Map;

import polyglot.ast.Formal;
import polyglot.ast.Node;

/**
 * A combination of an advice name and a vector parameters (variable names).
 * @author Eric Bodden
 */
public interface AdviceNameAndParams extends Node {

	/**
	 * Returns the advice name.
	 */
	public String getName();
	
	/**
	 * Returns the list of variable names. Variables that start with {@link AdviceDependency#WILDCARD}
	 * are wildcards.
	 */
	public List<String> getParams();

	/**
	 * If no parameters were given by the user (i.e. {@link #getParams()} returns an empty list),
	 * this method sets default parameters from the given map. The new AST node is returned. This
	 * node remains untouched.
	 */
	public AdviceNameAndParams defaultParams(Map<AdviceName, List<Formal>> adviceNameToFormals);

	/**
	 * Extracts the formals for this advice name from the map.
	 */
	public List<Formal> findFormalsForAdviceName(Map<AdviceName, List<Formal>> adviceNameToFormals);

	/**
	 * Returns <code>true</code>, if no parameters were given by the user and hence they are inferred
	 * from advice declarations.
	 */
	public boolean hasInferredParams();

}
