package abc.da.types;

import java.util.List;
import java.util.Map;
import java.util.Set;

import polyglot.ast.Formal;
import polyglot.types.SemanticException;
import abc.aspectj.types.AspectType;
import abc.da.ast.AdviceDependency;
import abc.da.ast.AdviceName;

/**
 * A type node for an aspect that can hold dependent advice.
 * @author Eric Bodden
 */
public interface DAAspectType extends AspectType {

	/**
	 * Checks that no two advice are given the same name in the aspect.
	 * @throws SemanticException if two advice with the same name exist
	 */
	public void checkDuplicateAdviceNames() throws SemanticException;

	/**
	 * Associates a mapping from advice names to their {@link Formal}s
	 * with this aspect type.
	 * @param adviceNameToFormals a mapping from advice names to {@link Formal}s of
	 * the respective advice; includes returning/throwing formal at the last position
	 */
	public void setAdviceNameToFormals(Map<AdviceName,List<Formal>> adviceNameToFormals);

	/**
	 * Returns the mapping from advice names to {@link Formal}s.
	 */
	public Map<AdviceName,List<Formal>> getAdviceNameToFormals();

	/**
	 * Adds a set of advice names that are references in a {@link AdviceDependency}.
	 * @param allAdviceNames
	 */
	public void addReferencedAdviceNames(Set<String> allAdviceNames);

	/**
	 * Returns the set of advice names that are references in a {@link AdviceDependency}.
	 */
	public Set<String> getAllReferencedAdviceNames();

}