package abc.da.types;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import polyglot.ast.Formal;
import polyglot.frontend.Source;
import polyglot.types.LazyClassInitializer;
import polyglot.types.SemanticException;
import polyglot.types.TypeSystem;
import abc.aspectj.types.AspectType_c;
import abc.da.ast.AdviceName;

/**
 * A type node for an aspect that can hold dependent advice.
 * @author Eric Bodden
 */
public class DAAspectType_c extends AspectType_c implements DAAspectType {

	protected Map<AdviceName,List<Formal>> adviceNameToFormals;
	protected Set<String> referencedAdviceNames;
	
	public DAAspectType_c(TypeSystem ts,
			LazyClassInitializer init, Source fromSource, int perKind) {
		super(ts,init,fromSource,perKind);
		adviceNameToFormals = new HashMap<AdviceName,List<Formal>>();
		referencedAdviceNames = new HashSet<String>();
	}
	
	/** 
	 * {@inheritDoc}
	 */
	public void checkDuplicateAdviceNames() throws SemanticException {
		for (AdviceName adviceName : adviceNameToFormals.keySet()) {
			for (AdviceName adviceName2 : adviceNameToFormals.keySet()) {
				if(adviceName==adviceName2)
					break;
				if(adviceName.getName().equals(adviceName2.getName())) {
					throw new SemanticException("Duplicate advice name: "+adviceName.getName(),adviceName.position());
				}
			}
		}
	}
	
	/** 
	 * {@inheritDoc}
	 */
	public void setAdviceNameToFormals(Map<AdviceName,List<Formal>> adviceNameToFormals) {
		this.adviceNameToFormals = adviceNameToFormals;
	}

	/** 
	 * {@inheritDoc}
	 */
	public Map<AdviceName,List<Formal>> getAdviceNameToFormals() {
		return adviceNameToFormals;
	}

	/** 
	 * {@inheritDoc}
	 */
	public void addReferencedAdviceNames(Set<String> adviceNames) {
		referencedAdviceNames.addAll(adviceNames);
	}
	
	/** 
	 * {@inheritDoc}
	 */
	public Set<String> getAllReferencedAdviceNames() {
		return new HashSet<String>(referencedAdviceNames);
	}

}
