package abc.da.types;

import java.util.List;
import java.util.Map;

import polyglot.ast.Formal;

import abc.aspectj.ast.AdviceDecl;
import abc.aspectj.types.AJContext;
import abc.da.ast.AdviceName;

/**
 * An extended context used for type checking of dependent advice.
 * @author Eric Bodden
 */
public interface DAContext extends AJContext {

	/**
	 * Adds an advice name and its {@link Formal}s to the context.
	 * returning/throwing formal has to be included (if given) at the last position
	 */
	public void addAdviceNameAndFormals(AdviceName adviceName, List<Formal> formals);

	/**
	 * Returns a mapping from an {@link AdviceName} to its {@link Formal}s in the current context. 
	 */
	public Map<AdviceName,List<Formal>> currentAdviceNameToFormals();

	/**
	 * Pushes the current advice declaration on the stack.
	 * Returns the resulting context.
	 */
	public AJContext pushAdviceDecl(AdviceDecl ad);

	/**
	 * Returns the current surrounding advice declaration.
	 */
	public AdviceDecl currentAdviceDecl();

}
