package abc.da.visit;

import java.util.List;
import java.util.Map;

import polyglot.ast.Formal;
import polyglot.ast.Node;
import polyglot.ast.NodeFactory;
import polyglot.frontend.Job;
import polyglot.types.TypeSystem;
import polyglot.visit.ContextVisitor;
import polyglot.visit.NodeVisitor;
import abc.da.ast.AdviceName;
import abc.da.ast.AdviceNameAndParams;
import abc.da.types.DAAspectType;
import abc.da.types.DAContext;

/**
 * This visitor invoked {@link AdviceNameAndParams#defaultParams(Map)} on each
 * {@link AdviceNameAndParams} object. This will set the default parameter
 * names if no such names were given by the user.
 * 
 * @author Eric Bodden
 */
public class PushParamNames extends ContextVisitor {

	public PushParamNames(Job job, TypeSystem ts, NodeFactory nf) {
		super(job, ts, nf);
	}
	
	@Override
	public Node leave(Node parent, Node old, Node n, NodeVisitor v) {
		if(n instanceof AdviceNameAndParams) {
			AdviceNameAndParams adviceNameAndParams = (AdviceNameAndParams) n;
			DAContext context = (DAContext) context();
			Map<AdviceName, List<Formal>> adviceNameToFormals =
				((DAAspectType)context.currentAspect()).getAdviceNameToFormals();
			n = adviceNameAndParams.defaultParams(adviceNameToFormals);
		}
		return super.leave(parent, old, n, v);
	}

}
