package abc.da.weaving.weaver.depadviceopt.ds;

import java.util.List;
import java.util.Map;

import soot.PointsToSet;

public class ShadowComparator {

	protected Map<String,List<String>> adviceNameToVars; 
	
	public ShadowComparator(Map<String, List<String>> adviceNameToVars) {
		this.adviceNameToVars = adviceNameToVars;
	}

	public boolean compatibleBindings(Shadow s1, Shadow s2) {
		assert s1.belongsToDependentAdvice();
		assert s2.belongsToDependentAdvice();
		
		String adviceName1 = s1.dependentAdviceName();
		String adviceName2 = s2.dependentAdviceName();
		List<String> filter1 = adviceNameToVars.get(adviceName1);
		List<String> filter2 = adviceNameToVars.get(adviceName2);
		assert filter1!=null && filter2!=null;
		
		//walk through the filter for the first shadow
		for(int i1 = 0; i1<filter1.size(); i1++) {
			//extract variable name
			String abstractVariableName1 = filter1.get(i1);
			//walk through the filter of the other shadows...
			for(int i2 = 0; i2<filter2.size(); i2++) {
				//...extracting the variable names there...
				String abstractVariableName2 = filter2.get(i2);
				//...and look for positions where we find the same variable
				if(abstractVariableName1.equals(abstractVariableName2)) {
					//extract both points-to sets (in terms of the original
					//variable names)
					PointsToSet pts1 = s1.pointsToSetOf(s1.variableNames().get(i1));
					PointsToSet pts2 = s2.pointsToSetOf(s2.variableNames().get(i2));
					//if they do not overlap, return false
					if(!pts1.hasNonEmptyIntersection(pts2)) {
						return false;
					}
				}
			}				
		}
		
		return true;
	}

}
