package abc.tm.weaving.weaver.tmanalysis;

import java.util.HashMap;
import java.util.Map;

import polyglot.util.ErrorInfo;
import soot.SootMethod;
import abc.da.weaving.weaver.depadviceopt.DependentAdviceFlowInsensitiveAnalysis;
import abc.main.Main;
import abc.tm.weaving.aspectinfo.TMAdviceDecl;
import abc.tm.weaving.aspectinfo.TMGlobalAspectInfo;
import abc.tm.weaving.aspectinfo.TraceMatch;
import abc.weaving.aspectinfo.AdviceDecl;

public class TMFlowInsensitiveAnalysis extends DependentAdviceFlowInsensitiveAnalysis {
	
	protected Map<String,TraceMatch> synchAdviceNameToTM;
	protected Map<String,TraceMatch> someAdviceNameToTM;
	protected Map<String,TraceMatch> symAdviceNameToTM;
	
	@Override
	public boolean analyze() {
		TMGlobalAspectInfo  gai = (TMGlobalAspectInfo) Main.v().getAbcExtension().getGlobalAspectInfo();

		synchAdviceNameToTM = new HashMap<String, TraceMatch>();
		for (TraceMatch tm : gai.getTraceMatches()) {
			String qualifiedName =
				tm.getSynchAdviceMethod().getDeclaringClass().getName() + "." + tm.getSynchAdviceMethod().getName();
			synchAdviceNameToTM.put(qualifiedName, tm);
		}

		someAdviceNameToTM = new HashMap<String, TraceMatch>();
		for (TraceMatch tm : gai.getTraceMatches()) {
			String qualifiedName =
				tm.getSomeAdviceMethod().getDeclaringClass().getName() + "." + tm.getSomeAdviceMethod().getName();
			someAdviceNameToTM.put(qualifiedName, tm);
		}

		symAdviceNameToTM = new HashMap<String, TraceMatch>();
		for (TraceMatch tm : gai.getTraceMatches()) {
			for (String sym : tm.getSymbols()) {
				SootMethod symbolAdviceMethod = tm.getSymbolAdviceMethod(sym);
				String qualifiedName =
					symbolAdviceMethod.getDeclaringClass().getName() + "." + symbolAdviceMethod.getName();
				symAdviceNameToTM.put(qualifiedName, tm);
			}
		}

		return super.analyze();
	}
	
	
	@Override
	protected void warn(AdviceDecl ad, int total, int disabled) {
		if(ad instanceof TMAdviceDecl) {
			TMAdviceDecl tmad = (TMAdviceDecl) ad;
			String msg = disabled + " out of " + total;
			
			String adviceName;
			TraceMatch tm;
			if(synchAdviceNameToTM.containsKey(tmad.getQualifiedAdviceName())) {
				adviceName = "synch-advice";
				tm = synchAdviceNameToTM.get(tmad.getQualifiedAdviceName());
			} else if(someAdviceNameToTM.containsKey(tmad.getQualifiedAdviceName())) {
				adviceName = "some-advice";
				tm = someAdviceNameToTM.get(tmad.getQualifiedAdviceName());
			} else if(symAdviceNameToTM.containsKey(tmad.getQualifiedAdviceName())) {
				tm = symAdviceNameToTM.get(tmad.getQualifiedAdviceName());
				String symbolName = null;
				for(String sym: tm.getSymbols()) {
					SootMethod symbolAdviceMethod = tm.getSymbolAdviceMethod(sym);
					if(tmad.getQualifiedAdviceName().endsWith(symbolAdviceMethod.getName())) {
						symbolName = sym;
						break;
					}
				}
				assert symbolName != null;
				adviceName = "symbol-advice of symbol '"+symbolName+"'";
			} else {
				throw new RuntimeException("unexpected tracematch advice in advice dependency! "+tmad.getQualifiedAdviceName());
			}
			
			Main.v().getAbcExtension().forceReportError(
					ErrorInfo.WARNING,
					msg + " shadows of " + adviceName + " for tracematch "+ tm.getContainerClass().getName()+"."+tm.getName()
					+" will not be executed because their dependencies are not fulfilled.",
					ad.getPosition()
			);
			
		} else {
			super.warn(ad, total, disabled);
		}
	}

}
